# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry, XmlNode
from fal_info import Results
#from datetime import datetime
import datetime
import time


class Migration(Command):
    try:
        from fal_info import FAL_MIGRATION_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_migration.py'

    """<migration>
        <job full="BOOL">
            <sourcelist>
                <source>
                    <name>HOSTNAME</name>
                    <passwd>PASSWORD</passwd>
                </source>
            </sourcelist>
            <schedule enable="BOOL">
                <date>
                    <year>YEAR</year>
                    <month>MONTH</month>
                    <day>DAY</day>
                    <hour>HOUR</hour>
                    <min>MINUTE</min>
                </date>
            </schedule>
        </job>
    <migration>"""

    def load(self):
        config = {
            'full': None,
            'source': {
                'name': None,
                'passwd': None,
            },
            'schedule': {
                'enable': None,
                'datetime': None,
            },
        }
        try:
            entry = self._load()
            job_node = entry.get().get_node('job')
            config['full'] = job_node.is_set_to_true('full')
            config['source']['name'] = job_node.get_value('sourcelist/source/name')
            config['source']['passwd'] = job_node.get_value('sourcelist/source/passwd')
            try:
                config['schedule']['enable'] = job_node.get_node('schedule').is_enabled()
            except:
                pass
            try:
                config['schedule']['datetime'] = datetime.datetime(
                        int(job_node.get_value('schedule/date/year')),
                        int(job_node.get_value('schedule/date/month')),
                        int(job_node.get_value('schedule/date/day')),
                        int(job_node.get_value('schedule/date/hour')),
                        int(job_node.get_value('schedule/date/minute')))
            except:
                pass
        except:
            pass
        return config

    def query(self):
        statusinfo = {
            'status': None,
            'starttime': None,
            'endtime': None,
            'hostname': None,
            'product': None,
            'elapsedtime': None,
            'progress': None,
        }
        try:
            entry = self._query()
            statusinfo_node = entry.get().get_node('statusinfo')
            statusinfo['status'] = statusinfo_node.get_value('status')
            try:
                statusinfo['starttime'] = datetime.datetime(*time.localtime(int(statusinfo_node.get_value('starttime/timestamp')))[:6])
            except:
                pass
            try:
                statusinfo['endtime'] = datetime.datetime(*time.localtime(int(statusinfo_node.get_value('endtime/timestamp')))[:6])
            except:
                pass
            statusinfo['hostname'] = statusinfo_node.get_value('host/name')
            statusinfo['product'] = statusinfo_node.get_value('product/name')
            try:
                statusinfo['elapsedtime'] = self.__sec2time(int(statusinfo_node.get_value('elapsedtime')))
            except:
                pass
            statusinfo['progress'] = statusinfo_node.get_value('progress')
        except:
            pass
        return statusinfo

    def __sec2time(self, seconds):
        minutes, seconds = divmod(seconds, 60)
        hours, minutes = divmod(minutes, 60)
        return "%02d:%02d:%02d" % (hours, minutes, seconds)
        #return datetime.time(hours, minutes, seconds)

    def apply(self, config, log=True, messages=None):
        _default_messages = {
            Results.OK: ['MSG_MIGRATION_CHANGE', {}],
            Results.AUTH_ERROR: ['ERR_MIGRATION_AUTH_CHECK', {}],
            None: ['ERR_MIGRATION_CHANGE', {}],
        }
        _default_messages.update(messages or {})

        entry = self._apply(config)
        result = getattr(entry, 'get_result', lambda: None)()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK])

    def execute(self, log=True, messages=None):
        _default_messages = {
            Results.OK: ['MSG_MIGRATION_EXEC_START', {}],
            None: ['ERR_MIGRATION_EXEC_START', {}],
        }
        _default_messages.update(messages or {})

        entry = self._execute()
        result = getattr(entry, 'get_result', lambda: None)()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK])

    def switch(self, log=True, messages=None):
        _default_messages = {
            Results.OK: ['MSG_MIGRATION_SWITCH_START', {}],
            None: ['ERR_MIGRATION_SWITCH_START', {}],
        }
        _default_messages.update(messages or {})

        entry = self._switch()
        result = getattr(entry, 'get_result', lambda: None)()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK])

    def list(self):
        sourcelist = []
        try:
            entry = self._list()
            sourcelist = [node.get_text() for node in
                    entry.get().get_node_list('sourcelist/source/name')]
        except:
            return sourcelist
        return sourcelist

    def check(self):
        switchable = True
        try:
            switchable = (self._check().get_result() in [Results.OK])
        except:
            pass
        return switchable

    def _load(self):
        entry = Entry('load', """<migration />""")
        return self.request(entry)

    def _save(self):
        entry = Entry('save', """<migration />""")
        return self.request(entry)

    def _query(self):
        entry = Entry('query', """<migration />""")
        return self.request(entry)

    def _apply(self, config):
        """
        config={
            full: True,
            'source': {
                'name': 'landisk-123456',
                'passwd': '654321',
            },
            'schedule': {
                'enabled': True,
                'datetime': datetime.datetime(2015, 1, 2, 3, 45),
            },
        }

        <migration>
            <job full="true">
                <soucelist>
                    <source>
                        <host>landisk-123456</host>
                        <passwd>654321</passwd>
                    </source>
                </soucelist>
                <schedule enable="true">
                    <date>
                        <year>2015</year>
                        <month>1</month>
                        <day>2</day>
                        <hour>3</hour>
                        <min>45</min>
                    </date>
                </schedule>
            </job>
        </migration>
        """
        entry = Entry('apply', """<migration>
            <job>
                <sourcelist>
                    <source />
                </sourcelist>
                <schedule>
                    <date />
                </schedule>
            </job>
        </migration>""")
        migration_node = entry.get()
        job_node = migration_node.get_node('job')
        job_node.set_bool_attr('full', config['full'])
        source_node = job_node.get_node('sourcelist/source')
        source_node.append_child('name', text=config['source']['name'])
        source_node.append_child('passwd', text=config['source']['passwd'])
        schedule_node = job_node.get_node('schedule')
        schedule_node.set_bool_attr('enable', config['schedule']['enable'])
        date_node = schedule_node.get_node('date')
        _datetime = config['schedule']['datetime']
        date_node.append_child('year', text=str(_datetime.year))
        date_node.append_child('month', text=str(_datetime.month))
        date_node.append_child('day', text=str(_datetime.day))
        date_node.append_child('hour', text=str(_datetime.hour))
        date_node.append_child('minute', text=str(_datetime.minute))
        return self.request(entry)

    def _execute(self):
        entry = Entry('execute', """<migration />""")
        return self.request(entry)

    def _switch(self):
        entry = Entry('switch', """<migration />""")
        return self.request(entry)

    def _status(self):
        entry = Entry('status', """<migration />""")
        return self.request(entry)

    def _list(self):
        entry = Entry('list', """<migration />""")
        return self.request(entry)

    def _check(self):
        entry = Entry('check', """<migration />""")
        return self.request(entry)


if __name__ == '__main__':
    migration = Migration()
    print migration.load()
    print migration.query()
